package com.ruoyi.ems.service;

import com.ruoyi.ems.domain.EmsFacs;
import com.ruoyi.ems.model.BoundaryObj;
import com.ruoyi.ems.model.QueryDevice;
import com.ruoyi.ems.model.TreeEntity;

import java.util.List;
import java.util.Map;

/**
 * 能源设施/系统Service接口
 *
 * @author ruoyi
 * @date 2024-07-10
 */
public interface IEmsFacsService {
    /**
     * 查询能源设施/系统
     *
     * @param id 能源设施/系统主键
     * @return 能源设施/系统
     */
    EmsFacs selectEmsFacsById(Long id);

    /**
     * 查询能源设施/系统
     *
     * @param code 能源设施/系统编码
     * @return 能源设施/系统
     */
    EmsFacs selectEmsFacsByCode(String code);

    /**
     * 查询能源设施/系统列表
     *
     * @param emsFacs 能源设施/系统
     * @return 能源设施/系统集合
     */
    List<EmsFacs> selectEmsFacsList(EmsFacs emsFacs);

    /**
     * 根据设施获取设备树结构
     *
     * @param queryDevice 查询参数
     * @return 设备树结构
     */
    List<TreeEntity> getFacsDevTree(QueryDevice queryDevice);

    /**
     * 根据设施类别获取绑定计量设备
     * @param category 设施类别
     * @param subCategory 设施子类别
     * @return 绑定计量设备列表
     */
    List<BoundaryObj> getFacsWithMeterDev(String areaCode, String category, String subCategory, Integer meterCls);

    /**
     * 新增能源设施/系统
     *
     * @param emsFacs 能源设施/系统
     * @return 结果
     */
    int insertEmsFacs(EmsFacs emsFacs);

    /**
     * 修改能源设施/系统
     *
     * @param emsFacs 能源设施/系统
     * @return 结果
     */
    int updateEmsFacs(EmsFacs emsFacs);

    /**
     * 批量删除能源设施/系统
     *
     * @param ids 需要删除的能源设施/系统主键集合
     * @return 结果
     */
    int deleteEmsFacsByIds(Long[] ids);

    /**
     * 删除能源设施/系统信息
     *
     * @param id 能源设施/系统主键
     * @return 结果
     */
    int deleteEmsFacsById(Long id);
}
